const {By, Key} = require('selenium-webdriver');
const assert = require('assert');
const {deleteSpaces, deleteHtmlTags, loadCurrentDataFiles, sleep, colors, clickInElementAtPercent, randomClickOnElement} = require("./utils");

let LABELS = [];
let LABELS_short_names = [];
let LABELS_descriptions = [];

/**
 * Check if the elements displayed correspond to those of the selected element.
 *
 * @param driver
 * @returns {Promise<void>}
 */
async function testCurrentLabels(driver) {
    try {
        let labelFromPapaya = await driver.findElement(By.css('#papayaViewer0')).getAttribute("label");
        if(labelFromPapaya == 0) {
            console.log(colors.bg.blue, "=> outside the image OR no label found", colors.reset);

        }else {
            let index = LABELS.indexOf(parseInt(labelFromPapaya));
            
            let nameContent = await driver.findElement(By.css('#labelName > h5')).getText();
            let descContent = await driver.findElement(By.id('labelDescription')).getAttribute("textContent");
            // if (index === -1) we do not care what it is displayed because the label is not in our list
            assert.equal(deleteSpaces(nameContent), deleteSpaces(LABELS_short_names[index]));
            assert.equal(deleteSpaces(descContent), deleteSpaces(deleteHtmlTags(LABELS_descriptions[index])));
            console.log(colors.bg.blue, "=> label [" + nameContent + "] passed", colors.reset);
        }
    }catch (e) {
        console.error(colors.bg.red, "ERROR :", colors.reset, " labels tests do not pass : \"", e.actual,"\" ", colors.bg.red, " != ", colors.reset, " \"", e.expected, "\"");
        console.log(e.message);
    }
}

exports.runLabelsTest = async function(driver, hash) {

    await driver.get("http://localhost:5000/preview?hash=" + hash).then(
        async () => {
            // Load current data
            let currentData = await loadCurrentDataFiles(driver, "../client/src/public/data", hash);
            LABELS = currentData[0];
            LABELS_short_names = currentData[1];
            LABELS_descriptions = currentData[2];

            // Tests for labels
            await driver.findElement(By.id('papayaViewer0')).then(async (viewer) => {
                await sleep(2000); // Wait a bit for the page to load properly

                // Test on existing elements
                await clickInElementAtPercent(driver, viewer, 10, 50); // label with description
                await testCurrentLabels(driver);
                await clickInElementAtPercent(driver, viewer, 35, 50); // labels with description
                await testCurrentLabels(driver);
                await clickInElementAtPercent(driver, viewer, 10, 55); // label without description
                await testCurrentLabels(driver);
                // Tests in the dark part
                await clickInElementAtPercent(driver, viewer, 10, 10); // nothing
                await testCurrentLabels(driver);
                await clickInElementAtPercent(driver, viewer, 60, 80); // nothing
                await testCurrentLabels(driver);
                // Test between the different parts of the viewer
                await clickInElementAtPercent(driver, viewer, 100, 100); // outside
                await testCurrentLabels(driver);
                await clickInElementAtPercent(driver, viewer, 65, 50); // outside
                await testCurrentLabels(driver);
                // Some randoms tests
                /*
                for (let i = 0; i < 15; i++) {
                    await randomClickOnElement(driver, viewer);
                    await testCurrentLabels(driver);
                }*/

                console.log(colors.bg.green, "Labels tests passed !", colors.reset);
            });
        }
    )
}

//Description button tests
async function clickOnDescriptionButton(driver) {
    const previewMenu = await driver.findElement(By.id('preview_menu'));
    const actions = driver.actions({async: true});
    await actions.move(previewMenu).perform();
    await actions.click().perform();
    await actions.keyDown(Key.ARROW_DOWN).keyDown(Key.ARROW_DOWN).perform(); //scroll to description button
    await driver.findElement(By.id('showDescription')).click();
}

exports.runDescriptionButtonTest = async function(driver, hash) {
    await driver.get("http://localhost:5000/preview?hash=" + hash).then(
        async () => {

            await driver.findElement(By.id('papayaViewer0')).then(async (viewer) => {
                await sleep(2000); // Wait a bit for the page to load properly

                await clickInElementAtPercent(driver, viewer, 10, 50); // label with description
                try {
                    const descriptionHide = await driver.findElement(By.id('labelDescription')).getText(); //empty because hide

                    await clickOnDescriptionButton(driver);
                    await sleep(1000); //wait a bit for animation hide description
                    let descriptionAfterClick = await driver.findElement(By.id('labelDescription')).getText(); //description show
                    assert.notEqual(deleteSpaces(descriptionHide), deleteSpaces(descriptionAfterClick));

                    await clickOnDescriptionButton(driver);
                    await sleep(1000); //wait a bit for animation hide description
                    descriptionAfterClick = await driver.findElement(By.id('labelDescription')).getText(); //empty because hide again
                    assert.equal(deleteSpaces(descriptionHide), deleteSpaces(descriptionAfterClick));
                    console.log(colors.bg.green, "Description button tests passed !", colors.reset);
                }catch(e) {
                    console.error(colors.bg.red, "ERROR :", colors.reset, " description button tests do not pass : \"", e.actual,"\" ", colors.bg.red, " != ", colors.reset, " \"", e.expected, "\"");
                    console.log(e.message);
                }
            });
        }
    );
}

 async function testInformationHide(driver, inCorrectLabel) {
    try {
        const eleSelected= await driver.findElement(By.id("labelDiv")).isDisplayed();
        assert.equal(eleSelected, inCorrectLabel);
        console.log(colors.bg.blue, "=> background hide passed", colors.reset);
    }catch (e) {
        console.error(colors.bg.red, "ERROR :", colors.reset, " background tests do not pass : \"", e.actual,"\" ", colors.bg.red, " != ", colors.reset, " \"", e.expected, "\"");
        console.log(e.message);
    }
}
exports.runBackgroundTest = async function(driver, hash) {
    await driver.get("http://localhost:5000/preview?hash=" + hash).then(
        async () => {

            await driver.findElement(By.id('papayaViewer0')).then(async (viewer) => {
                await sleep(2000); // Wait a bit for the page to load properly
                // Check if informations is hidden when crossbar is in dark part
                await clickInElementAtPercent(driver, viewer, 10, 10); // nothing
                await testInformationHide(driver, false);
                await clickInElementAtPercent(driver, viewer, 35, 50); // labels with description
                await testInformationHide(driver, true);


            });
        }
    );
}