const {By, until} = require('selenium-webdriver');

/**
 * CONSTANTS
 */

/**
 * Set of colors for the prints
 *
 * reset: reset current color,
 * fg: change the text color,
 * bg: change the color of the highlighting
 */
exports.colors = {
    reset: "\x1b[0m",
    fg: {
        black: "\x1b[30m",
        red: "\x1b[31m",
        green: "\x1b[32m",
        yellow: "\x1b[33m",
        blue: "\x1b[34m",
        magenta: "\x1b[35m",
        cyan: "\x1b[36m",
        white: "\x1b[37m",
        crimson: "\x1b[38m"
    },
    bg: {
        black: "\x1b[40m",
        red: "\x1b[41m",
        green: "\x1b[42m",
        yellow: "\x1b[43m",
        blue: "\x1b[44m",
        magenta: "\x1b[45m",
        cyan: "\x1b[46m",
        white: "\x1b[47m",
        crimson: "\x1b[48m"
    }
};

exports.KEYNAME_NUMPAD = ["\uE01B", "\uE01C", "\uE01D", "\uE01E", "\uE01F", "\uE020", "\uE021", "\uE022", "\uE023"];

// Accessible via the loadCurrentDataFiles function
let LABELS = [];
let LABELS_short_names = [];
let LABELS_descriptions = [];

let inputJSON = undefined;

/**
 * FUNCTIONS
 */

/**
 * Get a random x >= min && x < max
 * @param min
 * @param max
 * @returns {*}
 */
exports.getRandomBetween = function (min, max) {
    return Math.random() * (max - min) + min;
}

/**
 * Removes spaces from the input string if it is not undefined
 * @param string
 * @returns {undefined|*} undefined if the string is undefined, the string without spaces otherwise
 */
exports.deleteSpaces = function(string) {
    if (string !== undefined) return string.replace(/\s/g, "");
    return undefined;
}

/**
 * Removes the html tags while keeping the content
 *
 * @param string
 * @returns {undefined|*}
 */
exports.deleteHtmlTags = function(string) {
    if (string !== undefined){
        let tags = string.match(/<\s*[a-z]*[^>]*>(.*?)<\s*\/\s*[a-z]*>/g); // Get all tags

        if (tags !== null)
            for (const tag of tags) { // keep only content
                let content = tag.slice(tag.indexOf(">") + 1);
                content = content.slice(0, content.indexOf("<"));
                string = string.replace(tag, content);
            }
        return string;
    }
    return undefined;
}

exports.sleep = function(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}

async function loadInputJson(directoryPath, hash) {
    if (inputJSON === undefined) {
        inputJSON = require(directoryPath + "/job" + hash + "/input.json");
    }
}

/**
 * Loads the data according to the selected segmentation
 *
 * @param driver
 * @param dataPath Path of the .json files
 * @returns {Promise<[][]>} A Promise containing 3 arrays (new data): [LABELS][LABELS_short_names][LABELS_descriptions]
 */
exports.loadCurrentDataFiles = async function (driver, dataPath, hash) {
    await loadInputJson(dataPath, hash);

    const selectedSegmentation = await driver.findElement(By.id('segmentation')).getAttribute("value");
    const selectedLayerJson = inputJSON.preview_options.find((layer) => layer.image === selectedSegmentation);

    //Getting mapping path from the input.json
    let mapping_fileName = selectedLayerJson.options.regions_mapping;

    // Loads labels from the mapping specified in the input (if exists/we find it)
    if (mapping_fileName !== "") {
        let dataLabels = require(dataPath + "/" + mapping_fileName);
        let dataDescriptions = require(dataPath + "/" + "labels_descriptions.json");
        LABELS = [];
        LABELS_short_names = [];
        LABELS_descriptions = []; // reset arrays
        for (let label in dataLabels) {
            LABELS.push(parseInt(label));
            LABELS_short_names.push(dataLabels[label]);
            LABELS_descriptions.push((dataDescriptions[dataLabels[label]] !== undefined) ? dataDescriptions[dataLabels[label]] : "Element without description");
        }
    }
    return [LABELS, LABELS_short_names, LABELS_descriptions];
}

exports.loadOverlaysData = async function(dataPath, hash) {
    await loadInputJson(dataPath, hash);
    const LAYERS = inputJSON.preview;
    const LAYERS_options = inputJSON.preview_options;
    let LAYERS_categories = [];
    let LAYERS_enabled = [];
    LAYERS_options.forEach(LAYER_option => {
        LAYERS_categories.push(LAYER_option.options.category);
        LAYERS_enabled.push(LAYER_option.options.enabled);
    })
    return [LAYERS, LAYERS_categories, LAYERS_enabled];
}

exports.waitLoadingPage = async function(driver, dataPath, hash) { 
    const layersData = await exports.loadOverlaysData(dataPath, hash);
    //wait the loading of the last overlay
    const nbLayers = layersData[0].length - 1;
    await driver.wait(until.elementLocated(By.id('ImageButton' + nbLayers +'0')), 30000, 'Timed out after 30 seconds', 1000);
}
/**
 * Perform a click in the given element at the x/y position
 *
 * @param driver
 * @param element
 * @param x
 * @param y
 * @returns {Promise<void>}
 */
 async function clickInElementAt(driver, element, x, y) {
    try {
        let elementSize = await element.getRect();

        if (x > element.width || y > element.height) throw new Error("x/y position is not in the element");

        let startPosX = await elementSize.x;
        let startPosY = await elementSize.y;

        const actions = driver.actions({async: true});
        // Performs mouse move action onto the element
        await actions.move({x:(parseInt(startPosX) + parseInt(x)),y:(parseInt(startPosY) + parseInt(y))}).perform();
        await actions.click().perform();

    }catch (e){
        console.error("Error on element click : " + e );
    }
}

/**
 * Click randomly in the given element
 *
 * @param driver
 * @param element
 * @returns {Promise<void>}
 */
exports.randomClickOnElement = async function(driver, element) {
    try {
        let elementSize = await element.getRect();

        let randomX = getRandomBetween(0, elementSize.width);
        let randomY = getRandomBetween(0, elementSize.height);

        await clickInElementAt(driver, element, randomX, randomY);

    }catch (e){
        console.error("Error on random element click : " + e );
    }
}

/**
 * Perform a click in the given element at the x/y position in percentage to avoid errors due to window resizing.
 *
 * @param driver
 * @param element
 * @param xPercent x in percent (0 <= x =< 100)
 * @param yPercent y in percent (0 <= y =< 100)
 * @returns {Promise<void>}
 */
exports.clickInElementAtPercent = async function(driver, element, xPercent, yPercent) {
    try {
        let elementSize = await element.getRect();

        await clickInElementAt(driver, element, (elementSize.width/100)*xPercent, (elementSize.height/100)*yPercent);

    }catch (e){
        console.error("Error on random element click : " + e );
    }
}