/**
 * Created on Sun Feb 20 2022
 *
 * @author Jérémie GRIMAL (jgrimal)
 */
const CURRENT_VERSION = 1;
const DATA_DIR = "./data"

const fs = require('fs');
const fetch = require('node-fetch-commonjs');
const JSDOM = require('jsdom').JSDOM;

/**
 * Two arrays :
 *      - LABELS_short_names : Names corresponding to the different parts of the brain. ( /!\ filled from the json files given in the data folder).
 *      - LABELS_descriptions: Descriptions corresponding to the different parts of the brain ( /!\ will be filled in at the execution of the program according to the LABELS_short_names).
 */
let LABELS_short_names = [];
let LABELS_descriptions = [];

/**
 * Redefinitions of incomplete or abbreviated words in LABELS_short_names to improve description search.
 */
const completesWords = {
    "inf.": "inferior",
    "sup.": "superior",
    "CSF": "cerebro spinal fluid",
    // ...
};

/**
 * Loads the label names of the DATA_DIR JSON files into the LABELS_short_names array
 *
 * @returns {Promise<void>}
 */
function loadLabelsNames() {
    // sync --> May be to review in asynchronous but for this task not very important
    fs.readdirSync(DATA_DIR).forEach(file => {
        let currentJSON = require(DATA_DIR + "/" + file);
        for (let label in currentJSON) {
            LABELS_short_names.push(currentJSON[label]);
        }
    });
    // Remove duplicates
    LABELS_short_names = [...new Set(LABELS_short_names)];
}

/**
 * Give the list LABELS_short_names in a better format for searching.
 * (e.g. with full words instead of an abbreviation).
 *
 * @returns {[]} An array of names in the same order as LABELS_short_names
 */
function getReworksLabelsNames() {
    console.log("REWORKING LABELS ...");
    let reworksNames = [];
    for (const nameIndex in LABELS_short_names) {
        for (const key in completesWords) {
            if(LABELS_short_names[nameIndex].includes(key)){
                reworksNames[nameIndex] = LABELS_short_names[nameIndex].replace(key, completesWords[key]);
            }
        }
        if (reworksNames[nameIndex] === undefined) reworksNames[nameIndex] = LABELS_short_names[nameIndex];
    }
    return reworksNames;
}

/**
 * Create a list of links to call the Mediawiki API, for a given list of labels.
 * The links will be used to search for existing Wikipedia pages associated with the labels.
 *
 * @param labelsNames Labels to generate links
 * @returns {[]} An array of links in the same order as the input (labelsNames)
 */
function createMediawikiApiUrls(labelsNames) {
    let urls = [];

    for (const nameIndex in labelsNames) {
        let originUrl = "https://en.wikipedia.org/w/api.php";

        let params = {
            action: "query",
            list: "search",
            srsearch: "brain " + labelsNames[nameIndex],
            format: "json"
        };

        originUrl = originUrl + "?origin=*";
        Object.keys(params).forEach(function (key) {
            originUrl += "&" + key + "=" + params[key];
        });

        urls[nameIndex] = originUrl;
    }
    return urls;
}

/**
 * Gives the Wikipedia title (keys) of existing pages (which can be used in the Wikipedia url to access the corresponding page).
 *
 * @param mediawikiApiUrls Ordered (same order as LABELS_short_names) list of urls to use to search for keys
 * @param labelsNames Ordered (same order as LABELS_short_names) list of label names
 * @returns {Promise<[]>} An array of promises corresponding to the list of keys (page titles) of existing Wikipedia pages
 */
async function getKeysOfExistingDescriptions(mediawikiApiUrls, labelsNames) {
    console.log("SEARCHING EXISTING DESCRIPTIONS ...");

    let descKeys = [];
    await Promise.all(Object.keys(mediawikiApiUrls).map((urlIndex) => fetch(mediawikiApiUrls[urlIndex])
        .then(function(response){return response.json();})
        .then(function(response) {
            words = labelsNames[urlIndex].split(" ");
            let found = false;
            for (const res of response.query.search) {
                for (const w of words) {
                    if (res.title.toLowerCase().includes(w.toLowerCase())){
                        found = true;
                        break;
                    }
                }
                if (found){
                    descKeys[urlIndex] = res.title;
                    console.log("vv - " + LABELS_short_names[urlIndex] + " --> " + res.title);
                    break;
                }
            }
            if (!found){
                descKeys[urlIndex] = undefined;
                console.log("xx - " + LABELS_short_names[urlIndex] + " --> " + "NOT FOUND !!!"); // TODO : See why some descriptions are not found when they exist
                if (labelsNames[urlIndex].includes("accumbens")) { // TODO : ex : see why it does not find for "accumbens" then remove
                    console.log(words);
                    console.log(response.query.search);
                }
            }
        })
        .catch(function(error){console.error(error);})));

    return descKeys;
}

/**
 * Retrieves Wikipedia pages from a list of keys (page titles used in the url) to find the list of corresponding descriptions.
 *
 * @param keysToSearchingOnWikipedia Ordered (same order as LABELS_short_names) list of keys (page titles) of existing Wikipedia pages
 * @returns {Promise<[]>} An array of promises corresponding to the list of descriptions for each label of LABELS_short_names
 */
async function fetchDescriptions(keysToSearchingOnWikipedia) {
    let descriptions = [];

    let selector = '.mw-parser-output > p:not(.mw-empty-elt)'
    let originUrl = 'https://en.wikipedia.org/wiki/';

    console.log("FETCHING DESCRIPTIONS ...");

    await Promise.all(Object.keys(keysToSearchingOnWikipedia).map(
        function (keyIndex) {
            if (keysToSearchingOnWikipedia[keyIndex] !== undefined) {
                return fetch(originUrl + keysToSearchingOnWikipedia[keyIndex])
                    .then(resp => resp.text())
                    .then(text => {
                        let dom = new JSDOM(text)
                        let {document} = dom.window;
                        let list = [...document.querySelectorAll(selector)].map(a => a.textContent)

                        const regexReference = /\[[0-9]+\]/g; // used to remove Wikipedia references like "[1]"
                        descriptions[keyIndex] = list[0].replaceAll("\n", " ").replaceAll("\"", "\\\"").replaceAll(regexReference, ""); // remove unnecessary values
                        descriptions[keyIndex] += "[source: <a href=\\\"" + originUrl + keysToSearchingOnWikipedia[keyIndex] + "\\\">Wikipedia</a>]."; // Adds the link to the Wikipedia page
                    })
            } else {
                descriptions[keyIndex] = "Element without description";
                return undefined;
            }
        }

    ));

    return descriptions;
}

/**
 * Formats global arrays (LABELS, LABELS_short_names and LABELS_descriptions) in JSON format.
 *
 * @returns {string} Global arrays in JSON format
 */
function formatJson() {
    console.log("FORMATTING TO JSON ...");

    let content = "{" + "\n";

    for (let i = 0; i < LABELS_short_names.length; ++i) {
        content += "\t\t" + "\"" + LABELS_short_names[i] + "\"" + ": " + "\"" + LABELS_descriptions[i] + "\"," + "\n";
    }

    content = content.slice(0, -2) + "\n" + "}";

    return content;
}

/**
 * Write the input JSON to a file named structures_mapping_Vx.json. // with x = CURRENT_VERSION
 *
 * @param jsonContent JSON to write
 */
function writeJson(jsonContent) {
    console.log("WRITING FILE ...");

    fs.writeFile('./structures_mapping_V' + CURRENT_VERSION + '.json', jsonContent, err => {
        if (err) {
            console.error(err);
            return;
        }
        console.log("File written successfully !");
    })
}

/**
 * Main function to run.
 *
 * @returns {Promise<void>}
 */
async function main() {
    await loadLabelsNames();

    let reworkedLabels = getReworksLabelsNames();

    LABELS_descriptions = await fetchDescriptions(await getKeysOfExistingDescriptions(createMediawikiApiUrls(reworkedLabels), reworkedLabels));

    writeJson(formatJson());
}

// Run
main();
